/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "ftp_main.h"

/* --------start struct message--------------------- */
void ftp_message_free(struct ftp_message* msg) {
	if (!msg) {
		return;
	}
	ftp_message_free(msg->next);
	free(msg->msg);
	/* set the status to uninitialized */
	msg->msg = (char*) 0;
	msg->code = 0;
	msg->error = 0;
	msg->error_type = ERROR_NONE;
	msg->errno_copy = 0;
	msg->from_fd = -1;
}

void ftp_message_strip_respcode(struct ftp_message *msg) {
	char* new = strip_respcodes(msg->msg);
	free(msg->msg);
	msg->msg = new;
}

struct ftp_message ftp_message_construct(int code, const char* msg) {
	struct ftp_message ret = { (char*) 0,      /* msg */
				   0,              /* code */
				   0,              /* error */
				   ERROR_NONE,     /* error_type */
				   0,              /* errno_copy */
				   -1,             /* from_fd */
	     (struct ftp_message*) 0               /* next */
	};

	ret.code = code;
	ret.msg = strdup(msg);
	enough_mem(ret.msg);

	return ret;
}

void ftp_message_setm(struct ftp_message* ftp_msg,
			int code, const char* msg,
			int error, int errno_copy,
			int from_fd, struct ftp_message* next) {

	ASSUME(ftp_msg != NULL);

	ftp_msg->code = code;
	ftp_msg->msg = strdup(msg);
	enough_mem(ftp_msg->msg);
	ftp_msg->error = error;
	ftp_msg->errno_copy = errno_copy;
	ftp_msg->from_fd = from_fd;
	ftp_msg->next = next;
}

struct ftp_message ftp_message_empty() {
	struct ftp_message ret = { (char*) 0,      /* msg */
				   0,              /* code */
				   0,              /* error */
				   ERROR_NONE,     /* error_type */
				   0,              /* errno_copy */
				   -1,             /* from_fd */
	     (struct ftp_message*) 0               /* next */
	};

	return ret;
}

/* ----------end struct message--------------------- */

