/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "jproxygw.h"

#define EVENTS_MAX 30

static struct {
	struct event_t event[EVENTS_MAX];
	int nevents;
} events;

void event_init(void) {
	memset(&events, 0, sizeof(events));
	events.nevents = 0;
}

void event_deinit(void) {
	int i;
	for (i = 0; i < events.nevents; i++) {
		free(events.event[i].description);
	}
	memset(&events, 0, sizeof(events));
	events.nevents = 0;
}

int event_register(int event, event_func_t event_func, char* description) {
	int i = events.nevents;
	if (i > EVENTS_MAX) {
		return -1;
	}

	events.event[i].event = event;
	events.event[i].event_func = event_func;
	events.event[i].description = strdup(description);

	events.nevents++;
	return 0;
}

int event_fire(int event, struct proxyinfo* pi) {
	int i;
	int ret;
	for (i = 0; i < events.nevents; i++) {
		if (events.event[i].event == event) {
			jlog(7, "Function got fired by event: %s",
						events.event[i].description);
			ret = (events.event[i].event_func)(pi);
			jlog(7, "Function (%s) returned value %d",
					events.event[i].description, ret);
			/* XXX evaluate ret? we would need at least two types
			 * of errors:
			 * - dramatic errors
			 * - non-dramatic errors */
		}
	}
	return 0;
}

