/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#ifndef __JPROXYGW_UTIL_H__
#define __JPROXYGW_UTIL_H__

#define ASSUME(x) check_assumption(x, "x", __FILE__, __LINE__)

struct proxyinfo; /* XXX */

/* functions that don't rely on any external function or type
 * definition */
void enough_mem(const void*);
void free_and_zero_char(char**);
void scnprintf (char *os, size_t len, const char *str, ...);
char* my_strcasestr(const char* haystack, const char* needle);
void reap_chld_info (int);
int checkbegin(const char*, const char*);
char* strip_respcodes(const char* msg);
int respcode(const char*);
unsigned long int setlastbits(int);
void toupstr(char*);
void char_squeeze(char *const, int);
char* extract_path(const char*);
char* extract_file(const char*);
void encrypt_password(void);
int cryptcmp(const char*, const char*);
char* to_ascii(char *, int *, int);
void term_before_newline(char* s);
void term_before_last_newline(char* s);
void replace_not_larger(char* s, char* replace_what, char* replace_with);
char* replace_larger(char* s, char* replace_what, char* replace_with);
char* char_prepend(const char*, const char*);
char* char_append(const char*, const char*);
char* char_enclose(const char*, const char*, const char*);
char* strnulldup(const char*);
char* prependcode(const char* s, int code, int is_last_msg);
char* escape_username(const char*);
char* unescape_username(const char*);
void overwrite_string(char*);

/* functions that depend on config.c */
int changeid(int, int, const char*, struct proxyinfo*);
int change_root(const char* stage, struct proxyinfo*);
int dropprivileges(const char* stage, struct proxyinfo*);

/* functions that rely on log() */
void check_assumption(int, const char*, const char*, int);
int check_say_timeout(int, char*, int);

struct ip_t parse_ip(const char*);
#define GET_IP_SERVER	0
#define GET_IP_CLIENT	1

unsigned long int get_uint_ip(int, struct proxyinfo*);
const char* get_char_ip(int, struct proxyinfo*);

const char* get_char_peer_ip(int);
unsigned long int get_uint_peer_ip(int);
int get_interface_ip(const char* iface, struct sockaddr_in*);
int get_interface_name(const struct sockaddr_in, char*);
const char* conv_ip2char(unsigned long int);
char* mconv_ip2char(unsigned long int);
char* mconv_int2char(signed int);
char* mconv_uint2char(unsigned int);
char* mconv_luint2char(unsigned long int);
char* mconv_lint2char(signed long int);
char* mconv_float2char(float);
const char* base_name(const char*);

#endif
