/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "tp_main.h"

#define CHUNKSIZE	8192

static int closed = -1;
void sighandler(int);

static
int pass_from_to(int fd1, int fd2,
		 char* buffer, size_t bufsize,
		 struct timeval* timeout) {

	fd_set readset;
	fd_set writeset;
	int ret;

	FD_ZERO(&readset);
	FD_SET(fd1, &readset);

	FD_ZERO(&writeset);
	FD_SET(fd2, &writeset);

	ret = select(MAX_VAL(fd1, fd2) + 1, &readset, NULL, NULL, timeout);
	if (ret == 0) {
		/* timeout */
	}
	if (ret < 0) {
		jlog(3, "select error: %s", strerror(errno));
		return -1;
	}
	/* XXX timeout ? */
	ret = read(fd1, buffer, bufsize);
	if (ret < 0) {
		jlog(3, "read error: %s", strerror(errno));
		return -1;
	}
	if (ret == 0) {
		shutdown(fd2, SHUT_WR);
		if (closed == -1) {
			/* the first */
			closed = fd1;
			return 1;
		} else if (closed == fd1) {
			/* the same again */
			return 1;
		} else {
			/* both */
			return 0;
		}
	}
	/* XXX timeout ? */
	ret = writeall(fd2, buffer, ret);
	if (ret < 0) {
		return -1;
	}
	if (ret == 0) {
		return 0;
	}
	return ret;
}

#define BUFSIZE 8192
static
int pass_data(int fd1, int fd2) {
	int ret;
	fd_set readset;
	char buffer[BUFSIZE];
	struct timeval timeout_orig = { 0, 0 };
	struct timeval timeout;

	timeout_orig.tv_sec = config_get_ioption("timeout", 300);

	while(1) {
		FD_ZERO(&readset);
		if (closed != fd1) {
			FD_SET(fd1, &readset);
		}
		if (closed != fd2) {
			FD_SET(fd2, &readset);
		}

		timeout = timeout_orig;
		ret = select(MAX_VAL(fd1, fd2) + 1, &readset, NULL, NULL,
								&timeout);
		if (ret <= 0) { break; }
		alarm(timeout.tv_sec);
		if (FD_ISSET(fd1, &readset)) {
			ret = pass_from_to(fd1, fd2, buffer, BUFSIZE,
								&timeout);
			if (ret <= 0) { break; }
		}
		if (FD_ISSET(fd2, &readset)) {
			ret = pass_from_to(fd2, fd1, buffer, BUFSIZE,
								&timeout);
			if (ret <= 0) { break; }
		}
		alarm(0);
	}
	alarm(0);
	close(fd1);
	close(fd2);
	return ret;
}

int handle_connection(struct proxyinfo* pi) {
	struct serverinfo* si = pi->si;
	struct sockaddr_in dest;
	struct sigaction sa;
	struct portrangestruct* source_portrange;
	unsigned long int source_addr;
	const char* sourceport_range = config_get_option("sourceportrange");

	source_portrange = config_parse_portranges(sourceport_range);
	source_addr = config_get_addroption("sourceaddress", INADDR_ANY);

	dest = si->connection.transparent_destination;

	/* XXX just for testing. It's an IP of google
	dest.sin_addr.s_addr = inet_addr("216.239.59.104");
	dest.sin_port = htons(80);
	*/

	si->connection.serversocket = openportiaddr(dest.sin_addr.s_addr,
						ntohs(dest.sin_port),
						source_addr,
						source_portrange,
						pi);

	if (source_portrange) {
		config_destroy_portrange(source_portrange);
	}

	if (si->connection.serversocket < 0) {
		return -1;
	}

	/* installing a signal handler prevents from killing the process after
	 * a SIGALRM is received */
	sa.sa_handler = sighandler;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	sigaction(SIGALRM, &sa, 0);

	pass_data(si->connection.clientsocket,
			si->connection.serversocket);

	return 0;
}

void sighandler(int signo) {

}
