/* 
 * Copyright (C) 1999-2004 Joachim Wieland <joe@mcknight.de>
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111, USA.
 */

#include "tp_main.h"
#ifdef HAVE_LIBWRAP
#include <tcpd.h>
#endif

static int child_setup(struct proxyinfo* pi);

struct config_directives_list main_configuration_directives_array[] = {
	{"access",		TAG_ALL, "deny"   , EM, WSP },
	{"changerootdir",	TAG_STARTUP, (char*) 0, EM, WSP },
	{"changeroot",		TAG_STARTUP, "never", EM, WSP },
	{"timeout",		TAG_ALL, "300", EM, WSP },
	{"debuglevel",		TAG_ALL, "7"      , EM, WSP },
	{"dropprivileges",	TAG_STARTUP, "start", EM, WSP },
	{"libwrapservicenames",	TAG_STARTUP, "ftp-gw jftpgw", EM, FL },
	{"limit",		TAG_CONNECTED, (char*) 0, EM, WSP },
	{"listen",		TAG_STARTUP, "0.0.0.0:2370", EM, FL },
	{"logfile",		TAG_ALL, "/var/log/jftpgw.log", EM, WSP },
	{"pidfile",		TAG_STARTUP, "/var/run/jftpgw.pid", EM, WSP },
	{"runasgroup",		TAG_STARTUP, (char*) 0, EM, WSP },
	{"runasuser",		TAG_STARTUP, (char*) 0, EM, WSP },
	{"sourceaddress",	TAG_ALL, (char*) 0, EM, WSP },
	{"sourceportrange",	TAG_ALL, (char*) 0, EM, WSP },
	{"syslogfacility",	TAG_ALL, "daemon", EM, WSP },
	{"throughput",		TAG_ALL, (char*) 0, EM, WSP },
	{"transparent-proxy",	TAG_CONNECTED, "yes", EM, WSP },
	{ (char*) 0,            0, (char*) 0, 0, 0 }
};

struct config_namespace_directives_list main_configuration_directives = {
	"main",       /* namespace name */
	main_configuration_directives_array
};

struct config_option_values_list main_configuration_options_array[] = {
	{"access",              {"allow", "deny", TERM} },
	/*                           <---- better                          */
	{"changeroot",          {"start", "startsetup", "connect",
				  "connectsetup", "loggedin", "never", TERM} },
	/*                           <---- better                          */
	{"dropprivileges",      {"start", "startsetup", "connect",
				  "connectsetup", "loggedin", "never", TERM} },
	{"syslogfacility", {
#ifdef HAVE_LOG_FACILITY_LOG_AUTH
				"auth",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_AUTHPRIV
				"authpriv",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_CRON
				"cron",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_DAEMON
				"daemon",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_FTP
				"ftp",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_KERN
				"kern",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL0
				"local0",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL1
				"local1",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL2
				"local2",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL3
				"local3",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL4
				"local4",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL5
				"local5",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL6
				"local6",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LOCAL7
				"local7",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_LPR
				"lpr",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_MAIL
				"mail",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_NEWS
				"news",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_SYSLOG
				"syslog",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_USER
				"user",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_UUCP
				"uucp",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_CONSOLE
				"console",
#endif
#ifdef HAVE_LOG_FACILITY_LOG_SECURITY
				"security",
#endif
				TERM } },
	{ TERM, { TERM } }
};

struct config_namespace_options_list main_configuration_options = {
	"main",
	main_configuration_options_array
};

int tp_add_configuration() {
	config_configuration_list_add(&main_configuration_directives);
	config_options_list_add(&main_configuration_options);
	return 0;
}

static
int child_setup(struct proxyinfo *pi) {
	int cs = pi->si->connection.clientsocket;

	if (stage_action("connectsetup", pi) < 0) {
		say(cs, "Error setting up (see logfile)\r\n");
		return -1;
	}
	return 0;
}

int tp_allocate(struct proxyinfo* pi) {
	struct clientinfo* ci = malloc(sizeof(struct clientinfo));
	/*
	struct serverinfo* si = pi->si;
	*/

	enough_mem(ci);
	memset(ci, 0, sizeof(struct clientinfo));

	pi->ci = ci;
	ci->service = "tp";

	return 0;
}

int tp_deallocate(struct proxyinfo* pi) {
	struct clientinfo* ci = pi->ci;
	struct serverinfo* si = pi->si;

	if (!ci) {
		/* if we didn't call proto_accept() (because we are the master
		 * process), there is no deallocation */
		return 0;
	}

	close(si->connection.clientsocket);
	close(si->connection.serversocket);

	free(ci);
	pi->ci = (struct clientinfo*) 0;
	return 0;
}

int tp_reject(struct proxyinfo* pi) {
	int cs = pi->si->connection.clientsocket;
	say(cs, "Too many connections, sorry\r\n");
	close(cs);
	return 0;
}

int tp_connect(struct proxyinfo* pi) {
	int cs = pi->si->connection.clientsocket;
	if (stage_action("connect", pi) < 0) {
		say(cs, "421 Error setting up (see logfile)\r\n");
		return -1;
	}
	return 0;
}

int tp_accept(struct proxyinfo* pi) {
	int ret;

	tp_allocate(pi);

	if (child_setup(pi) < 0) {
		return -1;
	}

	/* Now we call the commandhandler to handle user commands */
	ret = handle_connection(pi);
	jlog(8, "Exited from handle_connection");
	return ret;
}

int tp_init_modules(void) {
	return 0;
}

int tp_reread_config_before(struct proxyinfo* pi) {
	return 0;
}

int tp_reread_config_after(struct proxyinfo* pi) {
	return 0;
}

